using System;
using System.Linq;
using UnityEngine;

public class CuboidGenerator : ShapeGenerator
{
    #region InspectorVariables
    [Header("Triangle values")]
    [SerializeField] protected float _ab = 1f;
    [SerializeField] protected float _bc = 1f;
    [SerializeField] protected Vector3 _height = new Vector3(0, 1.0F, 0);
    public float ab
    {
        get { return _ab; }
        set
        {
            _ab = value;
            GenerateShapeForAll();
        }
    }
    public float bc
    {
        get { return _bc; }
        set
        {
            _bc = value;
            GenerateShapeForAll();
        }
    }
    public Vector3 height
    {
        get { return _height; }
        set
        {
            _height = value;
            GenerateShapeForAll();
        }
    }
    #endregion InspectorVariables

    #region Implementation
    protected override (Vector3[] vertices, int[] triangles) GenerateTopology()
        => CreateCuboid(ab, bc, height);

    /// <summary>
    /// Creates circle vertecies and triangles around the midPoint at (0,0,0)
    /// </summary>
    /// <param name="points"></param>
    /// <param name="invert"></param>
    /// <returns></returns>
    public static (Vector3[], int[]) CreateCuboid(float ab, float bc, Vector3 height, Vector3 offset, bool invert = false)
    {
        Vector3[] bottom_edge = GetRectanglePoints(ab, bc);
        Vector3[] bottom_vertices =
            new Vector3[] { }
            .AppendRange(bottom_edge)   
            .Append(bottom_edge[0])
            .ToArray();
        int[] bottom_boundary = Enumerable.Range(1, bottom_edge.Length).ToArray();

        Vector3[] top_edge = GetRectanglePoints(ab, bc, -height);
        Vector3[] top_vertices =
            new Vector3[] { }
            .AppendRange(top_edge)
            .Append(top_edge[0])
            .ToArray();
        int[] top_boundary = Enumerable.Range(1, top_edge.Length).ToArray();

        return CreatePrism(
                    CreatePlane(top_vertices),
                    CreatePlane(bottom_vertices, true),
                    top_boundary,
                    bottom_boundary
                );
    }

    public static (Vector3[], int[]) CreateCuboid(float ab, float bc, Vector3 height, bool invert = false)
        => CreateCuboid(ab, bc, height, Vector3.zero, invert);
    #endregion Implementation
}
