using System;
using System.Linq;
using UnityEngine;

public class CylinderGenerator : ShapeGenerator
{
    #region InspectorVariables
    [Header("Triangle values")]
    [SerializeField] protected int npoints = 100;
    [SerializeField] protected Vector3 _height = new Vector3(0, 0, 1.0F);
    
    [SerializeField] protected float _radius = 1f;

    public float radius
    {
        get { return _radius; }
        set
        {
            _radius = value;
            GenerateShapeForAll();
        }
    }
    public Vector3 height
    {
        get { return _height; }
        set
        {
            _height = value;
            GenerateShapeForAll();
        }
    }
    #endregion InspectorVariables

    #region Implementation
    protected override (Vector3[] vertices, int[] triangles) GenerateTopology()
        => CreateCylinder(radius, npoints, height);

    /// <summary>
    /// Creates circle vertecies and triangles around the midPoint at (0,0,0)
    /// </summary>
    /// <param name="points"></param>
    /// <param name="invert"></param>
    /// <returns></returns>
    public static (Vector3[], int[]) CreateCylinder(float radius, int npoints, Vector3 height, Vector3 offset, bool invert = false)
    {
        Vector3[] bottom_edge = GetCirclePoints(radius, npoints);
        Vector3[] bottom_vertices =
            new Vector3[] { }
            .AppendRange(bottom_edge)   
            .Append(bottom_edge[0])
            .ToArray();
        int[] bottom_boundary = Enumerable.Range(1, bottom_edge.Length).ToArray();

        Vector3[] top_edge = GetCirclePoints(radius, npoints, -height);
        Vector3[] top_vertices =
            new Vector3[] { }
            .AppendRange(top_edge)
            .Append(top_edge[0])
            .ToArray();
        int[] top_boundary = Enumerable.Range(1, top_edge.Length).ToArray();

        return CreatePrism(
                    CreatePlane(top_vertices),
                    CreatePlane(bottom_vertices, true),
                    top_boundary,
                    bottom_boundary
                );
    }

    public static (Vector3[], int[]) CreateCylinder(float radius, int npoints, Vector3 height, bool invert = false)
        => CreateCylinder(radius, npoints, height, Vector3.zero, invert);
    #endregion Implementation
}
