using System;
using System.Linq;
using UnityEngine;

public class PyramidGenerator : ShapeGenerator
{
    #region InspectorVariables
    [Header("Triangle values")]
    [SerializeField] protected float _ab = 1f;
    [SerializeField] protected float _bc = 1f;
    [SerializeField] protected Vector3 _d_offset = new Vector3(0.5f, 0.5f, -1f);
    public float ab
    {
        get { return _ab; }
        set
        {
            _ab = value;
            GenerateShapeForAll();
        }
    }
    public float bc
    {
        get { return _bc; }
        set
        {
            _bc = value;
            GenerateShapeForAll();
        }
    }
    public Vector3 d_offset
    {
        get { return _d_offset; }
        set
        {
            _d_offset = value;
            GenerateShapeForAll();
        }
    }
    #endregion InspectorVariables

    #region Implementation
    protected override (Vector3[] vertices, int[] triangles) GenerateTopology()
        => CreatePyramid(ab, bc, d_offset);

    /// <summary>
    /// Creates circle vertecies and triangles around the midPoint at (0,0,0)
    /// </summary>
    /// <param name="points"></param>
    /// <param name="invert"></param>
    /// <returns></returns>
    public static (Vector3[], int[]) CreatePyramid(float ab, float bc, Vector3 d_offset, Vector3 offset, bool invert = false)
    {
        Vector3[] bottom_edge = GetRectanglePoints(ab, bc);
        Vector3[] bottom_vertices =
            new Vector3[] { }
            .AppendRange(bottom_edge)
            .Append(bottom_edge[0])
            .ToArray();
        int[] bottom_boundary = Enumerable.Range(1, bottom_edge.Length).ToArray();

        Vector3[] top_edge = GetRectanglePoints(0f, 0f, d_offset);
        Vector3[] top_vertices =
            new Vector3[] { }
            .AppendRange(top_edge)
            .Append(top_edge[0])
            .ToArray();
        int[] top_boundary = Enumerable.Range(1, top_edge.Length).ToArray();

        return CreatePrism(
                    CreatePlane(top_vertices),
                    CreatePlane(bottom_vertices, true),
                    top_boundary,
                    bottom_boundary
                );
    }

    public static (Vector3[], int[]) CreatePyramid(float ab, float bc, Vector3 d_offset, bool invert = false)
        => CreatePyramid(ab, bc, d_offset, Vector3.zero, invert);
    #endregion Implementation
}
