using System;
using System.Linq;
using UnityEngine;

public class TriangleGenerator : ShapeGenerator
{
    #region InspectorVariables
    [Header("Triangle values")]
    [SerializeField] protected float _ab = 1f;
    [SerializeField] protected float _height = 1f;
    [SerializeField] protected float _c = 0.5f;
    public float ab
    {
        get { return _ab; }
        set
        {
            _ab = value;
            GenerateShapeForAll();
        }
    }
    public float height
    {
        get { return _height; }
        set
        {
            _height = value;
            GenerateShapeForAll();
        }
    }
    public float c
    {
        get { return _c; }
        set {
            _c = value;
            GenerateShapeForAll();
        }
    }
    #endregion InspectorVariables

    #region Implementation
    protected override (Vector3[] vertices, int[] triangles) GenerateTopology()
        => CreateTriangle(ab, height, c);

    /// <summary>
    /// Creates circle vertecies and triangles around the midPoint at (0,0,0)
    /// </summary>
    /// <param name="points"></param>
    /// <param name="invert"></param>
    /// <returns></returns>
    public static (Vector3[], int[]) CreateTriangle(float ab, float height, float c, Vector3 offset, bool invert = false)
    {
        Vector3[] edge = GetTrianglePoints(ab, height, c);
        Vector3[] vertices =
            new Vector3[] { offset }
            .AppendRange(edge)
            .Append(edge[0])
            .ToArray();

        return CreatePlane(vertices, invert);
    }

    public static (Vector3[], int[]) CreateTriangle(float ab, float height, float c, bool invert = false)
        => CreateTriangle(ab, height, c, Vector3.zero, invert);
    #endregion Implementation
}
