using System;
using System.Linq;
using UnityEngine;

public class TriangluarPrismGenerator : ShapeGenerator
{
    #region InspectorVariables
    [Header("Triangle values")]
    [SerializeField] protected float _triangle_ab = 1f;
    [SerializeField] protected float _triangle_height = 1f;
    [SerializeField] protected float _triangle_c = 0.5f;
    [SerializeField] protected float _prism_height = 1f;
    public float triangle_ab
    {
        get { return _triangle_ab; }
        set
        {
            _triangle_ab = value;
            GenerateShapeForAll();
        }
    }
    public float triangle_height
    {
        get { return _triangle_height; }
        set
        {
            _triangle_height = value;
            GenerateShapeForAll();
        }
    }
    public float triangle_c
    {
        get { return _triangle_c; }
        set
        {
            _triangle_c = value;
            GenerateShapeForAll();
        }
    }
    public float prism_height
    {
        get { return _prism_height; }
        set
        {
            _prism_height = value;
            GenerateShapeForAll();
        }
    }
    #endregion InspectorVariables

    #region Implementation
    protected override (Vector3[] vertices, int[] triangles) GenerateTopology()
        => CreateTriangluarPrism(triangle_ab, triangle_height, triangle_c, prism_height);

    /// <summary>
    /// Creates circle vertecies and triangles around the midPoint at (0,0,0)
    /// </summary>
    /// <param name="points"></param>
    /// <param name="invert"></param>
    /// <returns></returns>
    public static (Vector3[], int[]) CreateTriangluarPrism(float triangle_ab, float triangle_height, float triangle_c, float prism_height, Vector3 offset, bool invert = false)
    {
        Vector3[] bottom_edge = GetTrianglePoints(triangle_ab, triangle_height, triangle_c);
        Vector3[] bottom_vertices =
            new Vector3[] { }
            .AppendRange(bottom_edge)
            .Append(bottom_edge[0])
            .ToArray();
        int[] bottom_boundary = Enumerable.Range(1, bottom_edge.Length).ToArray();

        Vector3[] top_edge = GetTrianglePoints(triangle_ab, triangle_height, triangle_c, new Vector3(0, 0, -prism_height));
        Vector3[] top_vertices =
            new Vector3[] { }
            .AppendRange(top_edge)
            .Append(top_edge[0])
            .ToArray();
        int[] top_boundary = Enumerable.Range(1, top_edge.Length).ToArray();

        return CreatePrism(
                    CreatePlane(top_vertices),
                    CreatePlane(bottom_vertices, true),
                    top_boundary,
                    bottom_boundary
                );
    }

    public static (Vector3[], int[]) CreateTriangluarPrism(float triangle_ab, float triangle_height, float triangle_c, float prism_height, bool invert = false)
        => CreateTriangluarPrism(triangle_ab, triangle_height, triangle_c, prism_height, Vector3.zero, invert);
    #endregion Implementation
}
